@@ -643,7 +643,7 @@ abstract class ExtLocations {
 
         @Override
         protected final void clear(DynamicObject store) {
-            UnsafeAccess.unsafePutObject(getArray(store, false), getOffset(), null, this);
+            setObjectInternal(store, null, false);
         }
 
         @Override
@@ -727,7 +727,7 @@ abstract class ExtLocations {
 
         @Override
         protected final void clear(DynamicObject store) {
-            UnsafeAccess.unsafePutObject(store, getOffset(), null, this);
+            setObjectInternal(store, null);
         }
 
         @Override
@@ -896,6 +896,11 @@ abstract class ExtLocations {
             }
         }
 
+        @Override
+        protected void clear(DynamicObject store) {
+            setIntInternal(store, 0);
+        }
+
         @Override
         public boolean canStore(Object value) {
             return value instanceof Integer;
@@ -979,6 +984,11 @@ abstract class ExtLocations {
             }
         }
 
+        @Override
+        protected void clear(DynamicObject store) {
+            setDoubleInternal(store, 0);
+        }
+
         @Override
         protected void set(DynamicObject store, Object value, boolean guard, boolean init) throws com.oracle.truffle.api.object.IncompatibleLocationException {
             if (canStore(value)) {
@@ -1168,6 +1178,11 @@ abstract class ExtLocations {
             setIntInternal(store, value, guard);
         }
 
+        @Override
+        protected void clear(DynamicObject store) {
+            setIntInternal(store, 0, false);
+        }
+
         @Override
         public boolean canStore(Object value) {
             return value instanceof Integer;
@@ -1239,6 +1254,11 @@ abstract class ExtLocations {
             return UnsafeAccess.unsafeGetFinalDouble(getArray(store, condition), getOffset(), condition, this);
         }
 
+        @Override
+        protected void clear(DynamicObject store) {
+            setDoubleInternal(store, 0, false);
+        }
+
         @Override
         public void setDouble(DynamicObject store, double value, boolean guard, boolean init) {
             if (!init) {
@@ -1333,6 +1353,11 @@ abstract class ExtLocations {
             return UnsafeAccess.unsafeGetFinalLong(store, getOffset(), condition, this);
         }
 
+        @Override
+        protected void clear(DynamicObject store) {
+            setLongInternal(store, 0L);
+        }
+
         @Override
         protected void set(DynamicObject store, Object value, boolean guard, boolean init) throws com.oracle.truffle.api.object.IncompatibleLocationException {
             if (canStore(value)) {
@@ -1424,6 +1449,11 @@ abstract class ExtLocations {
             UnsafeAccess.unsafePutLong(getArray(store, guard), getOffset(), value, this);
         }
 
+        @Override
+        protected void clear(DynamicObject store) {
+            setLongInternal(store, 0L, false);
+        }
+
         protected long getFinalLong(DynamicObject store, boolean condition) {
             return UnsafeAccess.unsafeGetFinalLong(getArray(store, condition), getOffset(), condition, this);
         }
